#!/usr/bin/env python
# coding: utf-8

# In[1]:


import math

def define_points(wid, hgt):
    '''Define the points.'''
    dy = hgt / 4
    dx = dy / math.sqrt(3)
    top_x = wid / 2
    top_y = -dy / 2
    points = [
        (top_x - dx, top_y + dy),
        (top_x + dx, top_y + dy),
        (top_x - 2 * dx, top_y + 2 * dy),
        (top_x - 0 * dx, top_y + 2 * dy),
        (top_x + 2 * dx, top_y + 2 * dy),
        (top_x - 3 * dx, top_y + 3 * dy),
        (top_x - 1 * dx, top_y + 3 * dy),
        (top_x + 1 * dx, top_y + 3 * dy),
        (top_x + 3 * dx, top_y + 3 * dy),
        (top_x - 2 * dx, top_y + 4 * dy),
        (top_x - 0 * dx, top_y + 4 * dy),
        (top_x + 2 * dx, top_y + 4 * dy),
    ]
    return points

def find_solutions(points):
    '''Find the solutions.'''
    solutions = []
    for i in range(len(points)):
        for j in range(i + 1, len(points)):
            dx_ij = points[j][0] - points[i][0]
            dy_ij = points[j][1] - points[i][1]
            dist_ij = math.sqrt(dx_ij * dx_ij + dy_ij * dy_ij)
            for k in range(j + 1, len(points)):
                dx_jk = points[k][0] - points[j][0]
                dy_jk = points[k][1] - points[j][1]
                dist_jk = math.sqrt(dx_jk * dx_jk + dy_jk * dy_jk)
                if math.isclose(dist_ij, dist_jk):
                    dx_ki = points[i][0] - points[k][0]
                    dy_ki = points[i][1] - points[k][1]
                    dist_ki = math.sqrt(dx_ki * dx_ki + dy_ki * dy_ki)
                    if math.isclose(dist_jk, dist_ki):
                        # This is a solution.
                        solutions.append((points[i], points[j], points[k]))
    return solutions


# In[2]:


import tkinter as tk

class EquilateralPuzzleApp:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title('equilateral_triangle_puzzle')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x400')

        # Build the UI.
        button = tk.Button(self.window, text='Solve', width=7, command=self.solve)
        button.pack(side=tk.TOP, pady=5)

        self.canvas = tk.Canvas(self.window, bg='white',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=5, side=tk.TOP, fill=tk.BOTH, expand=True)

        self.num_solutions_label = tk.Label(self.window)
        self.num_solutions_label.pack(side=tk.BOTTOM)

        # If the user presses Enter, solve.
        self.window.bind('<Return>', lambda event: self.solve())

        self.window.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def solve(self):
        # Define the points.
        self.num_solutions_label['text'] = ''
        canvas_wid = self.canvas.winfo_width()
        canvas_hgt = self.canvas.winfo_height()

        # Define the points.
        self.points = define_points(canvas_wid, canvas_hgt)

        # Find the solutions.
        self.solutions = find_solutions(self.points)

        # Draw the points and solutions.
        self.draw()

    def draw(self):
        '''Draw the points and solutions.'''
        # Delete any previous drawing objects.
        self.canvas.delete(tk.ALL)
        self.current_solution = None

        # Draw the points.
        RADIUS = 5
        COLOR = 'blue'
        for point in self.points:
            self.canvas.create_oval(
                point[0] - RADIUS, point[1] - RADIUS,
                point[0] + RADIUS, point[1] + RADIUS,
                fill=COLOR)

        # Draw the solutions starting with the first one.
        self.DELAY_MS = 250
        self.window.after(self.DELAY_MS, self.draw_next_solution, 0)

    def draw_next_solution(self, solution_number):
        '''Draw solution number solution_number.'''
        # Delete previous solutions.
        if self.current_solution is not None:
            self.canvas.delete(self.current_solution)

        # See if we've drawn them all.
        if solution_number == len(self.solutions):
            # We're done. Draw them all.
            for solution in self.solutions:
                self.draw_solution(solution)
            self.num_solutions_label['text'] = f'{len(self.solutions)} solutions'
        else:
            # Draw the next solution.
            self.draw_solution(self.solutions[solution_number])

            # Wait and then draw the net solution.
            self.window.after(self.DELAY_MS, self.draw_next_solution, solution_number + 1)

    def draw_solution(self, solution):
        self.current_solution = self.canvas.create_polygon(solution, outline='red', fill='')


# In[3]:


EquilateralPuzzleApp()
print('Done')


# In[ ]:




